@@ -14,24 +14,20 @@
  * limitations under the License.
  */
 
-#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 /**
- * @file module.c
- * GSSAPI module activation code
- *
- * $RCSfile: gridmap.c,v $
- * $Revision: 1.30.14.5 $
- * $Date $
+ * @file gss_assist/source/gridmap.c
+ * @brief GSS Assist Gridmap Functions
  */
-#endif
 
 #include "globus_i_gss_assist.h"
 #include "globus_gsi_system_config.h"
 #include "globus_gsi_credential.h"
 #include "globus_callout.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
+
 typedef struct _gridmap_line_s {
   char *dn;
   char **user_ids;
@@ -41,11 +37,11 @@ typedef struct _gridmap_line_s {
 #define QUOTING_CHARS			"\""
 #define ESCAPING_CHARS			"\\"
 #define COMMENT_CHARS			"#"
-/* Characters seperating user ids in the gridmap file */
+/* Characters separating user ids in the gridmap file */
 #define USERID_SEP_CHARS		","
 /*
  * Characters that terminate a user id in the gridmap file. This
- * is a combination of whitespace and seperators.
+ * is a combination of whitespace and separators.
  */
 #define USERID_TERMINATOR_CHARS		USERID_SEP_CHARS WHITESPACE_CHARS
 
@@ -58,7 +54,7 @@ typedef struct _gridmap_line_s {
 
 /*
  * Number of user id slots to allocate at a time
- * Arbitraty value, but must be >= 2.
+ * Arbitrary value, but must be >= 2.
  */
 #define USERID_CHUNK_SIZE		4
 
@@ -117,9 +113,9 @@ globus_l_gss_assist_gridmap_lookup(
   
 static
 globus_result_t
-globus_l_gss_assist_line_length(
+globus_l_gss_assist_read_line(
     FILE *                              fp,
-    size_t *                            len);
+    char **                             line);
 
 #endif
 
@@ -297,7 +293,7 @@ gridmapdir_newlease(char *     encodedglobusidp,
 
      while ((gridmapdirentry = readdir(gridmapdirstream)) != NULL)
      {
-       /* we dont want any files that dont look like acceptable usernames */
+       /* we don't want any files that don't look like acceptable usernames */
        if ((*(gridmapdirentry->d_name) == '%') || 
            (strcmp(gridmapdirentry->d_name, "root") == 0))   continue;
        else if (*(gridmapdirentry->d_name) == '.')           continue;
@@ -310,7 +306,7 @@ gridmapdir_newlease(char *     encodedglobusidp,
        sprintf(userfilename, "%s/%s", gridmapdir, gridmapdirentry->d_name);
        stat(userfilename, &statbuf);
        
-       if (statbuf.st_nlink == 1) /* this one isnt leased yet */
+       if (statbuf.st_nlink == 1) /* this one isn't leased yet */
        {   
            ret = link(userfilename, encodedfilename);
            free(userfilename);
@@ -520,7 +516,7 @@ gridmapdir_userok(char *     globusidp,
 
 /**
  * @brief Look up the default mapping for a Grid identity in a gridmap file
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  * 
  * @details
  * The globus_gss_assist_gridmap() function parses the default gridmap file
@@ -671,7 +667,7 @@ globus_gss_assist_gridmap(
 
 /**
  * @brief Gridmap entry existence check
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  *
  * @details
  * The globus_gss_assist_userok() function parses the default gridmap file
@@ -746,7 +742,7 @@ globus_gss_assist_userok(
         GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
             result,
             GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP,
-            (_GASL("The gridmap is malformated.  No user id's could be be found.")));
+            (_GASL("The gridmap is malformed.  No user id's could be be found.")));
         goto exit;
     }
 
@@ -770,7 +766,7 @@ globus_gss_assist_userok(
     GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
         result,
         GLOBUS_GSI_GSS_ASSIST_ERROR_USER_ID_DOESNT_MATCH,
-        (_GASL("The user id: %s, doesn't match the the DN: %s, in the "
+        (_GASL("The user id: %s, doesn't match the DN: %s, in the "
          "gridmap file: %s"),
          globusid,
          userid,
@@ -802,7 +798,7 @@ globus_gss_assist_userok(
 
 /**
  * @brief Look up the default Grid identity associated with a local user name
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  *
  * @details
  * The globus_gss_assist_map_local_user() function parses the 
@@ -940,11 +936,6 @@ globus_gss_assist_map_local_user(
 /* globus_gss_assist_map_local_user() */
 
 #ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
-
-/**
- * @name Gridmap Find DN
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * Locate the entry for the given DN in the default gridmap file
@@ -1019,35 +1010,14 @@ globus_i_gss_assist_gridmap_find_dn(
 
     do
     {
-        size_t                          line_len;
         char *                          line;
 
-        result = globus_l_gss_assist_line_length(gmap_stream, &line_len);
-        if (result != GLOBUS_SUCCESS || line_len == 0)
+	result = globus_l_gss_assist_read_line(gmap_stream, &line);
+        if (result != GLOBUS_SUCCESS || line == NULL)
         {
             break;
         }
 
-        line = malloc(++line_len);
-        if (line == NULL)
-        {
-            result = globus_error_put(globus_error_wrap_errno_error(
-                GLOBUS_GSI_GSS_ASSIST_MODULE,
-                errno,
-                GLOBUS_GSI_GSS_ASSIST_ERROR_ERRNO,
-                __FILE__,
-                _function_name_,
-                __LINE__,
-                _GASL("Could not allocate enough memory")));
-            break;
-        }
-
-        if (fgets(line, line_len, gmap_stream) == NULL)
-        {
-            free(line);
-	    break;		/* EOF or error */
-        }
-
         result = globus_i_gss_assist_gridmap_parse_line(line, &gline_tmp);
 	if (result != GLOBUS_SUCCESS)
 	{
@@ -1093,12 +1063,7 @@ globus_i_gss_assist_gridmap_find_dn(
     return result;
 } 
 /* gridmap_find_dn() */
-/* @} */
 
-/**
- * @name Find Local User
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * Locate the first entry with the given local user as the default in the
@@ -1108,7 +1073,7 @@ globus_i_gss_assist_gridmap_find_dn(
  * @param local_user
  *        the name to search for
  * @param gline
- *        the resulting gridmap_line_t contianing the user and DN information
+ *        the resulting gridmap_line_t containing the user and DN information
  *
  * @return
  *        0 on success, otherwise an error object identifier is returned.
@@ -1172,36 +1137,15 @@ globus_i_gss_assist_gridmap_find_local_user(
 
     do
     {
-        size_t                          line_len;
         char *                          line;
         char *                          save_line;
 
-        result = globus_l_gss_assist_line_length(gmap_stream, &line_len);
-        if (result != GLOBUS_SUCCESS || line_len == 0)
+        result = globus_l_gss_assist_read_line(gmap_stream, &line);
+        if (result != GLOBUS_SUCCESS || line == NULL)
         {
             break;
         }
         
-        line = malloc(++line_len);
-        if (line == NULL)
-        {
-            result = globus_error_put(globus_error_wrap_errno_error(
-                GLOBUS_GSI_GSS_ASSIST_MODULE,
-                errno,
-                GLOBUS_GSI_GSS_ASSIST_ERROR_ERRNO,
-                __FILE__,
-                _function_name_,
-                __LINE__,
-                _GASL("Could not allocate enough memory")));
-            break;
-        }
-        
-        if (fgets(line, line_len, gmap_stream) == NULL)
-        {
-            free(line);
-	    break;		/* EOF or error */
-        }
-
         save_line = strdup(line);
         
 	result = globus_i_gss_assist_gridmap_parse_line(line, &gline_tmp);
@@ -1227,7 +1171,7 @@ globus_i_gss_assist_gridmap_find_local_user(
             if(strcmp(local_user, *useridp) == 0)
             {
                 /* check all names, but only stop looking if we match a 
-                 * default name.  save the first nondefault match til 
+                 * default name.  Save the first non-default match until 
                  * we've checked all the defaults */
                 if(*useridp == gline_tmp->user_ids[0])
                 {
@@ -1281,12 +1225,7 @@ globus_i_gss_assist_gridmap_find_local_user(
     return result;
 }
 /* gridmap_find_local_user() */
-/* @} */
 
-/**
- * @name Gridmap Parse Line
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * 
@@ -1429,7 +1368,7 @@ globus_i_gss_assist_gridmap_parse_line(
     line = dn_end + 1;
     line += strspn(line, WHITESPACE_CHARS);
 
-    /* Parse list of unix user ID seperated by USERID_SEP_CHARS */
+    /* Parse list of unix user ID separated by USERID_SEP_CHARS */
     while (*line != NUL)
     {
 	int					userid_len;
@@ -1533,12 +1472,7 @@ globus_i_gss_assist_gridmap_parse_line(
     return result;
 } 
 /* gridmap_parse_line() */
-/* @} */
 
-/**
- * @name globus_i_gsi_gss_assist
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * Frees all memory allocated to a gridmap_line_t structure.
@@ -1582,12 +1516,7 @@ globus_i_gss_assist_gridmap_line_free(
     }
 } 
 /* gridmap_free_gridmap_line() */
-/* @} */
 
-/**
- * @name Gridmap Parse Globusid
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * Given a pointer to a string containing the globusid from the
@@ -1757,19 +1686,14 @@ globus_i_gss_assist_gridmap_parse_globusid(
     return result;
 } 
 /* gridmap_parse_globusid() */
-/* @} */
 
-/**
- * @name Hexadecimal Digit to Integer
- */
-/* @{ */
 /**
  * @ingroup globus_i_gsi_gss_assist
  * Convert an ascii character representing a hexadecimal digit
  * into an integer.
  *
  * @param xdigit
- *        character contianing the hexidecimal digit
+ *        character containing the hexadecimal digit
  *
  * @return
  *        the value in the xdigit, or -1 if error
@@ -1791,14 +1715,12 @@ globus_i_gss_assist_xdigit_to_value(
     return -1;
 } 
 /* xdigit_to_value() */
-/* @} */
-
 #endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
 
 
 /**
  * @brief Look up all Grid IDs associated with a local user ID
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  *
  * @details
  * The globus_gss_assist_lookup_all_globusid() function parses a 
@@ -1848,7 +1770,6 @@ globus_gss_assist_lookup_all_globusid(
     int *                                       dn_count)
 {
     char *                                      line;
-    size_t                                      line_len;
     int                                         i;
     int                                         max_ndx = 512;
     int                                         ndx = 0;
@@ -1904,32 +1825,12 @@ globus_gss_assist_lookup_all_globusid(
 
     while (!feof(gmap_stream))
     {
-        res = globus_l_gss_assist_line_length(gmap_stream, &line_len);
-        if (res != GLOBUS_SUCCESS || line_len == 0)
+        res = globus_l_gss_assist_read_line(gmap_stream, &line);
+        if (res != GLOBUS_SUCCESS || line == NULL)
         {
             break;
         }
 
-        line = malloc(++line_len);
-        if (line == NULL)
-        {
-            res = globus_error_put(globus_error_wrap_errno_error(
-                GLOBUS_GSI_GSS_ASSIST_MODULE,
-                errno,
-                GLOBUS_GSI_GSS_ASSIST_ERROR_ERRNO,
-                __FILE__,
-                _function_name_,
-                __LINE__,
-                _GASL("Could not allocate enough memory")));
-            break;
-        }
-
-        if (fgets(line, line_len, gmap_stream) == NULL)
-        {
-            free(line);
-            break;
-        }
-
         res = globus_i_gss_assist_gridmap_parse_line(line, &gline);
 
         if(res == GLOBUS_SUCCESS &&
@@ -1982,7 +1883,7 @@ globus_gss_assist_lookup_all_globusid(
 
 /**
  * @brief Authorize the peer of a security context to use a service
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  *
  * @details
  * The globus_gss_assist_map_and_authorize() function attempts to authorize
@@ -2154,7 +2055,7 @@ globus_gss_assist_map_and_authorize(
 
 /**
  * @brief Authorize a particular credential for shared access.
- * @ingroup globus_gsi_gss_assist
+ * @ingroup globus_gss_assist_gridmap
  *
  * @details
  * The globus_gss_assist_map_and_authorize_sharing() function attempts to
@@ -2563,87 +2464,66 @@ release_peer_name_buffer:
     return result;
 }
 
-/**
- * Determine length of the next line on the file stream
- *
- * Scans the input stream to determine the length of the next line
- * ending with \n or the length until the end of the file. The
- * value is returned in the integer pointed to by @a len. If the file
- * pointer is currently at end-of-file, *len will be set to 0.
- * 
- * @param fp
- *     File pointer to inspect
- * @param len
- *     Pointer to be set to the length
- *
- * @retval GLOBUS_SUCCESS
- *     Success
- * @retval GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP
- *     Error with gridmap
- */
 static
 globus_result_t
-globus_l_gss_assist_line_length(
-    FILE *                              fp,
-    size_t *                            len)
+globus_l_gss_assist_read_line(
+    FILE *				stream,
+    char **				line)
 {
+    char *                              buffer = NULL;
+    size_t                              line_len = 0;
+    size_t                              buffer_size = 0;
     globus_result_t                     result = GLOBUS_SUCCESS;
-    fpos_t                              pos;
-    int                                 line_len = -1;
-    int                                 rc;
     static char *                       _function_name_ =
-        "globus_l_gss_assist_line_length";
+        "globus_l_gss_assist_read_line";
 
-    *len = 0;
-
-    rc = fgetpos(fp, &pos);
-    if (rc < 0)
+    do
     {
-        GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
-            result,
-            GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP,
-            (_GASL("Couldn't determine position in file.")));
-        goto fgetpos_failed;
-    }
+        char * tmp;
 
-    rc = fscanf(fp, "%*[^\n]%*1[\n]%n", &line_len);
-    if (line_len == -1 && rc == 0)
-    {
-       /* match failure; see if we have an empty line */
-       rc = fscanf(fp, "%*1[\n]%n", &line_len);
-    }
+        tmp = realloc(buffer, buffer_size + BUFSIZ);
+        if (!tmp)
+        {
+            result = globus_error_put(globus_error_wrap_errno_error(
+                GLOBUS_GSI_GSS_ASSIST_MODULE,
+                errno,
+                GLOBUS_GSI_GSS_ASSIST_ERROR_ERRNO,
+                __FILE__,
+                _function_name_,
+                __LINE__,
+                _GASL("Could not allocate enough memory")));
+            goto fail;
+        }
 
-    if (rc < 0 && !feof(fp))
-    {
-        GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
-            result,
-            GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP,
-            (_GASL("Couldn't determine end of line in file.")));
-        goto fscanf_failed;
-    }
-    else if (feof(fp))
-    {
-        /* Assume end-of-file without newline */
-        clearerr(fp);
-        line_len = ftell(fp);
-    }
+        buffer = tmp;
+        buffer_size += BUFSIZ;
 
-    rc = fsetpos(fp, &pos);
-    if (rc < 0)
-    {
-        GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
-            result,
-            GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP,
-            (_GASL("Couldn't set position in file.")));
-        goto fsetpos_failed;
-    }
-
-    *len = line_len;
-
-fsetpos_failed:
-fscanf_failed:
-fgetpos_failed:
+        if (fgets(buffer + line_len, buffer_size - line_len, stream) == NULL)
+        {
+            if (! feof(stream))
+            {
+                result = globus_error_put(globus_error_wrap_errno_error(
+                    GLOBUS_GSI_GSS_ASSIST_MODULE,
+                    errno,
+                    GLOBUS_GSI_GSS_ASSIST_ERROR_ERRNO,
+                    __FILE__,
+                    _function_name_,
+                    __LINE__,
+                    _GASL("Could not read line")));
+                free(buffer);
+                buffer = NULL;
+            }
+            else if (line_len == 0)
+            {
+                free(buffer);
+                buffer = NULL;
+            }
+            goto fail;
+        }
+        line_len = strlen(buffer);
+    } while (buffer[line_len-1] != '\n');
+fail:
+    *line = buffer;
     return result;
 }
-/* globus_l_gss_assist_line_length() */
 #endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
